function b = db(x,p) 

  // Codage du dcimal vers le binaire sur p bits.
  // On passera en paramtre le nombre en base 10(x) que l'on veut
  // coder et l'entier(p) dsignant le nombre de bits souhait.

  b = zeros(1,p);
  if x>=2^p then error('Dimension'); end

  for i = 1:p
    d = x-2^(p-i);
    b(i) = (d>=0)*1;
    x = x-2^(p-i)*b(i);
  end

endfunction

//================================================================//

function table = Table(n,m,x)

  // Cette fonction gre la cration de la table contenant les bits
  // de sorties possibles .Elle retourne une matrice de dimension
  // (2^(m-1)=le nombre d'tat, 2*n=2*nb de sorties du codeur) .Il
  // faut placer en paramtre le nombre de sorties du codeur, le
  // nombre d'tages du registre et les polynomes gnrateurs en
  // binaire sous forme de matrice : chaque ligne correspondant  un
  // polynome.

  table = zeros(2^(m-1),2*n);
  E = zeros(2^(m-1),m-1);

  for i = 1:size(table,'r')
    E(i,:) = db(i-1,m-1);
  end
  // E contient tous les tats possibles

  for i = 1:size(table,'r'), for j = 1:n
    table(i,j) = modulo([0 E(i,:)]*x(j,:)',2);
    table(i,n+j) = modulo([1 E(i,:)]*x(j,:)',2);
  end, end

endfunction

//================================================================//

// Algorithme de dcodage (Viterbi)
function mess = convol_decoder_soft(code,n,m,x)

  // Cette fonction permet de dcoder un code en suivant
  // l'algorithme de Viterbi.
  // Il faut pour cela passer en paramtre le mot code en binaire,
  // puis le nombre de sorties (n) du codeur convolutionnel,
  // le nombre d'tages du registre (m) du codeur convolutionnel,
  // et enfin la matrice contenant les polynomes gnrateurs en
  // binaire(chaque ligne correspondant  un polynome)

  // Test de validit des paramtres
  // Nous ne testons pas si le code est binaire !
  init_encoder(0,n,m,x);

  // Dclaration
  compteur = 1;
  // Fentre de dcodage : 6 fois la longueur de contrainte
  f = 6*m;
  // L est la longueur du message original avant le codeur
  L = floor(length(code)/n);

  d = zeros(2^(m-1),2);
  // 2^(m-1) est le nombre d'etats
  // La matrice d va stocker  chaque fois les 2^m poids possibles 

  Dist = zeros(2^(m-1),f+1);
  Dist(:,1) = [0;10000*ones(2^(m-1)-1,1)];
  // La matrice Dist va stocker dans chaque colonne le poids des
  // diffrents tats ; ces poids sont retenus  chaque tour de boucle
  // par l'valuation de la matrice d.

  P = zeros(2^(m-1),f);
  // La matrice P va contenir le numro de l'tat prcdent qui a men
  //  l'tat actuel dans la matrice (la position dans une des
  // colonnes de P donne l'tat actuel), l'tat(00) est not(1) et
  // ainsi de suite.

  M = zeros(2^(m-1),f-1);
  // M est une matrice "intermdiaire" ncessaire  la rvaluation de
  // P : limination des chemins qui "s'arrtent"

  mess = zeros(1,L);

  // construction de matrices utiles pour les calculs de l'algorithme

  Jd = [zeros(f+1,1) eye(f+1,f+1)];
  Jd = Jd(:,1:f+1);
  Jp = [zeros(f,1) eye(f,f)];
  Jp = Jp(:,1:f);
  // Jd et Jp sont les matrices identits dcales d'une colonne vers
  // la droite.

  I = 1:2^(m-1);
  I = matrix(I,2,2^(m-2));
  I = [I I];
  // Par exemple pour m=3 on a I=[1 3 1 3;2 4 2 4] 
  // et pour m=4 I=[1 3 5 7 1 3 5 7;2 4 6 8 2 4 6 8].
  // Cette matrice sert pour retrouver l'tat prcdent et remplir P

  // Cration de la table contenant les bits de sorties possibles.
  table_sortie = Table(n,m,x);

  code_etat = [zeros(1,2^(m-2)) ones(1,2^(m-2))]

  code = [code zeros(1,f*L-length(code))];

  t = size(table_sortie,'r');

  for i = 1:f

    // On compare les bits du code avec les bits de la table.
    for j = 1:2, for k = 1:t,
        diffCode = (code((i-1)*n+1:i*n) - ...
                    table_sortie(k,(j-1)*n+1:j*n));
        d(k,j) = sum(diffCode.^2) + Dist(k,i);
      end
    end

    // Pour chaque tat on garde le chemin arrivant le plus petit
    dbis = matrix(d,2,t);
    [Min,ind] = min(dbis,'r');
    Dist(:,i+1) = Min';

    // P contient le numro de l'tat prcedent qui a permis d'arriver  l'tat prsent
    for l = 1:size(Dist,'r')
      P(l,i) = I(ind(l),l);
    end,

  end

  // on limine les chemins qui "s'arrtent" au fur et  mesure
  for i = 1:f-1
    for j = 1:size(P,'r')
      if P(j,f-i+1) ~= 0 then
        M(P(j,f-i+1),f-i) = 1;
      end
    end
    P(:,f-i) = P(:,f-i).*M(:,f-i);
  end

  z = 1;
  Z = zeros(z,2);
  // On regarde les positions restantes au dbut de la fentre (6 fois
  // la longueur de contrainte).
  for j = 1:size(P,'r')
    if P(j,1) ~= 0 then
      Z(z,:) = [j Dist(j,2)];
      z = z+1;
    end
  end

  // On garde celui de poids le plus faible.
  [Min,ind] = min(Z(:,2));
  p = ind;

  // Suivant sa position on en dduit le bit du message qui avait t
  // cod.
  mess(compteur) = code_etat(Z(p,1));

  winId  =  waitbar('Dcodage squence');
  ref = round(L/100);

  while compteur<L,

    // On reste dans la boucle tant que le message n'est pas complet

    if modulo(compteur,ref) == 0
      waitbar(round(compteur/L*100)/100,winId);
    end

    M = zeros(2^(m-1),f-1);

    // On dcale les matrices Dist et P d'une colonne vers la gauche
    Dist = Dist*Jd';
    P = P*Jp';

    // Puis on calcule les dernieres colonnes de ces 2 matrices

    for j = 1:2,
      for k = 1:t,
        diffCode = (code((f+compteur-1)*n+1:(f+compteur)*n) - ...
                    table_sortie(k,(j-1)*n+1:j*n));
        d(k,j) = sum(diffCode.^2) + Dist(k,f);
      end 
    end

    dbis  = matrix(d,2,t);
    [Min,ind] = min(dbis,'r');
    Dist(:,f+1) = Min';
    
    for l = 1:size(Dist,'r')
      P(l,f) = I(ind(l),l);
    end,

    // On limine les chemins qui "s'arrtent"
    for i = 1:f-1
      for j = 1:size(P,'r')
        if P(j,f-i+1) ~= 0 then
          M(P(j,f-i+1),f-i) = 1;
        end
      end
      P(:,f-i) = P(:,f-i).*M(:,f-i);
    end

    // On regarde les positions restantes au dbut de la fentre (6 fois
    // la longueur de contrainte).
    z = 1.
    Z = zeros(z,2);
    for j = 1:size(P,'r')
      if P(j,1) ~= 0 then
        Z(z,:) = [j Dist(j,2)];
        z = z+1;
      end
    end
  
    // On garde celui de poids le plus faible
    [Min,ind] = min(Z(:,2));
    p = ind;

    // Suivant sa position on en dduit le bit du message qui avait t
    // cod.
    mess(compteur+1) = code_etat(Z(p,1));

    // On incrmente le compteur de passage dans la boucle
    compteur = compteur+1;

  end

  winclose(winId);

endfunction

