// Performance des codes R=1/2 avec le dcodage de Viterbi
// avec dcision douce. Rapports signal  bruit trs faibles.

// Attention !!
// Pour l'instant ce script prend un peu de temps
// (4 minutes sur SunBlade 150). La fonction convol_decoder est
// "couteuse".

// Suppression de toutes les variables non protges
clear();
// Suppression de toutes les fentres graphiques
xdel(winsid());

getf("../convol_encoder.sci");
getf("../convol_decoder_soft.sci");

// Caractristique du codeur

// Nombre de sorties
n = 2;
// Nombre d'tages du registre
m = 4;
// Polynomes gnrateurs du codeur en binaire.
// Chaque ligne correspondant  un polynome.
x = [1 1 1 1;1 1 0 1]; // 17,15
// Polynomes gnrateurs du codeur en octal.
codeName = [17;15]; 
// Rendement
R = 1/n;

// Gnration d'un nom de fichier de sauvegarde.
fileName = sprintf('r_%i_%i_g_',1,n);
for i = codeName'; fileName = sprintf('%s%i_',fileName,i); end
fileName = sprintf('%ssl.mem',fileName);

// Nombre de bits  coder
a = 1000000;
// Le nombre d'itrations
compteur = 1;

// SNR in dB
EbN0_dB = [0:0.5:4];
printf('Avancement: ');

// The waveform used at the output of the encoder is U/NRZ
// The mean power of the signal
Ps = 1/2;
// The mean energy per symbol, the period is T
T = 1;
Es = Ps*T;
// The mean energy per useful bit at the input of the encoder
Eb = Es/R;

// EbN0 = Es/(N0*R) = (Es*1/T)/(N0*1/T*R) = Ps/(Pb*R).
// Pb = (standard deviation)^2 = Ps/(EbN0*R).

// We have to subtract 3 dB to the power of the noise since we do not
// simulate the matched filter : Pb = Ps/(2*EbN0*R).

// Usually the performances of a code are computed with a P/NRZ
// waveform transmission. U/NRZ is 3 dB lower in performances than
// P/NRZ, so we have to subtract 3 dB again to be able to compare our
// results with the ones published : Pb = Ps/(4*EbN0*R).

for SNR = EbN0_dB

  // SNR in linear
  EbN0 = 10^(SNR/10);

  compt = 0;
  erreur = 0;

  div = 10;

  while compt<div
    donnees = round(rand(1,a/div,'uniform'));
    code = convol_encoder(donnees,n,m,x);

    bruit = sqrt(Ps/(4*EbN0*R))*rand(1,length(code),'normal');
    code_bruit = code + bruit;

    mess = convol_decoder_soft(code_bruit,n,m,x);
    mess = mess(1:a/div);
    erreur = sum(modulo(donnees+mess,2))+erreur;
    compt = compt+1;
  end
  
  erreur = erreur/a;
  TEB(compteur) = erreur;
  printf('%i/%i ',compteur,length(EbN0_dB));
  compteur = compteur+1;

  save(fileName,EbN0_dB,TEB);

  // Affichage
  xset("font size",2);
  plot2d('nl',EbN0_dB(1:compteur-1),TEB,2);
  titre = sprintf('Performance code convolutif/decodage de Viterbi ');
  titre = sprintf('%s soft-decision (1/%i) (%i,%i), K=%i',titre,n,...
                  codeName(1),codeName(2),m);
  xtitle(titre,'Eb/N0','TEB');
  xgrid(1);

end

halt(); xdel();

